#! /bin/sh

pump_output()
{
    # read until 'start failed|monitoring'
    local line
    local str
    echo "start pumping" 
    while read line
    do
        str=${line/"start"//}
        if [ "$str" == "$line" ]; then 
            # no change, continue read
            echo "[SKIP] $line" 
        else
            str=${line/"failed"//}
            if [ "$str" == "$line" ]; then 
                # no change, success
                echo "[READY] $line" 
                return 0
            else
                # fail
                echo "[FATAL] $line" 
                return 1
            fi
        fi
    done <&6
    return 1 
}

pass_cnt=0
fail_cnt=0

# $1: expect
verify_output()
{
    local expect="$1"
    local line=""
    read -r line <&6 # -r: stop transform '\'
    local str=${line/"$expect"//}
    if [ "$str" == "$line" ]; then 
        # no change, character string not find.
        echo "[FAIL] $line" 
        fail_cnt=$(($fail_cnt+1))
        return 1
    else
        echo "[PASS] $line" 
        pass_cnt=$(($pass_cnt+1))
        return 0
    fi
}


# $1: dir
do_test_case()
{
    local dir="$1"
    # pump out until known rdc-demo start ok or fail
    pump_output
    if [ $? -ne 0 ]; then 
        echo "pump failed" 
        return 1
    fi

    pass_cnt=0
    fail_cnt=0
    # do real test here
    if [ $is_macos -ne 0 ]; then 
        # eat root dir create notify:
        #verify_output "add dir" 
        # have been avoided by sleep 1

        # start dirty work
        # for rdc-demo, which using AsyncIOService directly
        source ./test_case_macos.sh
    else
        # windows
        source ./test_case_windows.sh
    fi

    # should cd after start demo, otherwise rdc-demo will complain no tmp exist..."
    cd "$dir"
    test_file_changes "test.txt"
    test_dir_changes_1
    test_dir_changes_2
    test_dir_changes_3
    test_case_insensitive 

    #echo -e "\n"
    echo ""
    echo "test done, $pass_cnt PASS, $fail_cnt FAIL"
    echo "press any key to exit..."

    # cd back
    cd ..
}

#########################################3
# main
if [ -z "$RDC_HOME" ]; then 
    echo "must define RDC_HOME to indicate test program root dir before run this script!"
    exit 1
fi

if [ ! -f "$RDC_HOME/rdc-demo" ]; then 
    echo "test program not exist, fatal error!"
    exit 1
fi

is_macos=0
os="${OSTYPE/"darwin"//}"
if [ "$os" != "$OSTYPE" ]; then 
    # darwin: macos
    is_macos=1
fi

dir="tmp"
# clear old files
rm *.fifo
rm -rf "$dir"

echo "init fifo"
fifo_name_out="$$.out.fifo"
mkfifo "$fifo_name_out"

# 6 - out
exec 6<>"$fifo_name_out"

echo "init dir: $dir"
mkdir "$dir"
if [ $is_macos -ne 0 ]; then 
    # eat dir created action, otherwise we need a 'add dir' verify on macOS...
    sleep 1
fi

echo "start work process"
do_test_case "$dir" &

# 6 - out
"$RDC_HOME/rdc-demo" "$PWD/$dir" >&6 # 2>&1 
#"$RDC_HOME/rdc-demo" "$PWD/$dir" | tee "$fifo_name_out" #>&6 # 2>&1 

echo "wait worker process..."
wait

echo "finished, close fifo"
exec 6<&-

echo "cleanup"
rm -rf "$dir"
rm "$fifo_name_out"
echo "all done"
