#! /bin/sh

# $1: file name to monitor
test_file_changes()
{
    local file="$1"
    local newfile="new.$file"

    touch "$file"
    echo "touch $file :" 
    # touch trigger 2 actions
    verify_output "add file" 
    verify_output "modify file"

    echo "first line" > "$file"
    echo "modify $file :" 
    # '>' trigger 2 actions
    verify_output "modify file" 
    verify_output "modify file" 

    echo "last line" >> "$file"
    echo "modify $file :" 
    # '>>' triger only 1
    verify_output "modify file"

    mv "$file" "$newfile"
    echo "move $file :" 
    verify_output "rename file"

    rm "$newfile"
    echo "remove $file :" 
    verify_output "remove file"
}

test_dir_changes_1()
{
    local dir="abc"
    local newdir="def"
    local file="a.txt"
    local newfile="b.txt"

    mkdir "$dir"
    echo "mkdir $dir :"
    verify_output "add dir"

    touch "$dir/$file"
    echo "touch $dir/$file :"
    # touch trigger 3 actions
    verify_output "add file" 
    verify_output "modify dir"
    verify_output "modify file"

    echo "first line" > "$dir/$file"
    echo "modify $dir/$file :" 
    # '>' trigger 2 actions
    verify_output "modify file" 
    verify_output "modify file" 

    echo "last line" >> "$dir/$file"
    echo "modify $dir/$file :" 
    # '>>' triger only 1
    verify_output "modify file"

    mv "$dir/$file" "$dir/$newfile"
    echo "move $dir/$file :" 
    verify_output "rename file"
    verify_output "modify dir"

    mv "$dir" "$newdir"
    echo "move $dir :"
    verify_output "rename dir"

    rm "$newdir/$newfile"
    echo "remove $newdir/$newfile :"
    verify_output "remove file"
    verify_output "modify dir"

    rmdir "$newdir"
    echo "remove $newdir :"
    verify_output "remove dir"
}

test_dir_changes_2()
{
    local dir1="a"
    local dir11="aa"
    local dir2="b"
    local dir22="bb"
    local file1="a1.txt"
    local file11="a2.txt"
    local file2="b1.txt"
    local file22="b2.txt"

    # setup dir1
    mkdir "$dir1"
    echo "mkdir $dir1 :"
    verify_output "add dir"

    mkdir "$dir1/$dir11"
    echo "mkdir $dir1/$dir11 :"
    verify_output "add dir"
    verify_output "modify dir"

    touch "$dir1/$file1"
    echo "touch $dir1/$file1 :"
    verify_output "add file" 
    verify_output "modify dir"
    verify_output "modify file"

    # prepare data
    echo "this is file1" >> "$dir1/$file1"
    echo "modify $dir1/$file1 :" 
    verify_output "modify file"

    echo "this is file11" >> "$dir1/$dir11/$file11"
    echo "create & modify $dir1/$dir11/$file11 :" 
    verify_output "add file"
    verify_output "modify dir"
    verify_output "modify file"
    verify_output "modify file"

    # setup dir2
    mkdir "$dir2"
    echo "mkdir $dir2 :"
    verify_output "add dir"

    mkdir "$dir2/$dir22"
    echo "mkdir $dir2/$dir22 :"
    verify_output "add dir"
    verify_output "modify dir"

    touch "$dir2/$file2"
    echo "touch $dir2/$file2 :"
    verify_output "add file" 
    verify_output "modify dir"
    verify_output "modify file"

    # test copy
    cp "$dir1/$file1" "$dir2/$file2"
    echo "cp $dir1/$file1 $dir2/$file2 :"
    # cp trigger 3 actions
    #verify_output "add file" 
    verify_output "modify file" 
    verify_output "modify file"

    cp "$dir1/$dir11/$file11" "$dir2/$dir22/$file22"
    echo "cp $dir1/$dir11/$file11 $dir2/$dir22/$file22 :"
    verify_output "add file" 
    verify_output "modify dir"
    verify_output "modify file"

    # append data
    echo "this is file2" >> "$dir2/$file2"
    echo "modify $dir2/$file2 :" 
    verify_output "modify file"

    echo "this is file22" >> "$dir2/$dir22/$file22"
    echo "modify $dir2/$dir22/$file22 :" 
    verify_output "modify file"

    # test cover
    mv "$dir2/$file2" "$dir1/$dir11/$file11"
    echo "move $dir2/$file2 $dir1/$dir11/$file11 :" 
    verify_output "remove file"
    verify_output "remove file"
    verify_output "add file"
    verify_output "modify dir"
    verify_output "modify dir"

    # test move
    mv "$dir1/$file1" "$dir2/$dir22/"
    echo "move $dir1/$file1 $dir2/$dir22 :"
    verify_output "remove file"
    verify_output "add file"
    verify_output "modify dir"
    verify_output "modify dir"

    # test delete file
    rm "$dir2/$dir22/$file1"
    echo "remove $dir2/$dir22/$file1 :"
    verify_output "remove file"
    verify_output "modify dir"

    rm "$dir2/$dir22/$file22"
    echo "remove $dir2/$dir22/$file22 :"
    verify_output "remove file"
    verify_output "modify dir"

    rm "$dir1/$dir11/$file11"
    echo "remove $dir1/$dir11/$file11 :"
    verify_output "remove file"
    verify_output "modify dir"

    # test movm dir
    mv "$dir1/$dir11" "$dir2/$dir22"
    echo "move $dir1/$dir11 $dir2/$dir22"
    verify_output "remove dir"
    verify_output "add dir"
    verify_output "modify dir"
    verify_output "modify dir"

    mv "$dir2" "$dir1"
    echo "move $dir2 $dir1"
    verify_output "remove dir"
    verify_output "add dir"
    verify_output "modify dir"

    # test delete dir
    rmdir "$dir1/$dir2/$dir22/$dir11"
    echo "remove $dir1/$dir2/$dir22/$dir11 :"
    verify_output "remove dir"
    verify_output "modify dir"

    rmdir "$dir1/$dir2/$dir22"
    echo "remove $dir1/$dir2/$dir22 :"
    verify_output "remove dir"
    verify_output "modify dir"

    rmdir "$dir1/$dir2"
    echo "remove $dir1/$dir2 :"
    verify_output "remove dir"
    verify_output "modify dir"

    rmdir "$dir1"
    echo "remove $dir1 :"
    verify_output "remove dir"
}

test_dir_changes_3()
{
    local dir0="b"
    local dir1="a"
    local dir2="aa"
    local file1="a1.txt"
    local file2="a2.txt"
    local file21="aa1.txt"
    local file22="aa2.txt"

    # setup dir tree for following test
    echo "prepare dir tree..."
    # windows choked on '/tmp', using ../ instead
    rm -rf "../$dir1" 2>&1 >/dev/null
    mkdir "../$dir1"
    mkdir "../$dir1/$dir2"
    echo "this is file1" >> "../$dir1/$file1"
    echo "this is file2" >> "../$dir1/$file2"
    echo "this is file21" >> "../$dir1/$dir2/$file21"
    echo "this is file22" >> "../$dir1/$dir2/$file22"

    # test move in
    mv "../$dir1" "$dir1"
    echo "mv ../$dir1 $dir1 :"
    # triggers only one notify, the dir iteself
    verify_output "add dir" 

    # test rename
    mv "$dir1" "$dir0"
    echo "move $dir1 to $dir0 :" 
    verify_output "rename dir"

    # test move out
    mv "$dir0" "../$dir1"
    echo "move $dir0 to ../$dir1 :"
    verify_output "remove dir"

    # test copy 
    cp -r "../$dir1" "$dir1"
    echo "copy ../$dir1 :"
    verify_output "add dir"  # a
    verify_output "add file" # a/a1.txt
    verify_output "modify dir" # a
    verify_output "modify file" # a/a1.txt
    verify_output "add file" # a/a2.txt
    verify_output "modify dir" # a
    verify_output "modify file" # a/a2.txt
    verify_output "add dir"  # a/aa
    verify_output "modify dir" # a
    verify_output "add file" # a/aa/a21.txt
    verify_output "modify dir" # a/aa
    verify_output "modify file" # a/aa/a21.txt
    verify_output "add file" # a/aa/a22.txt
    verify_output "modify dir" # a/aa
    verify_output "modify file" # a/aa/a22.txt

    # test delete
    rm -rf "$dir1"
    echo "remove $dir1 :"
    verify_output "remove file" # a/a1.txt
    # sometimes combined two notify into one
    verify_output "modify dir" # a
    verify_output "remove file" # a/a2.txt
    verify_output "modify dir" # a
    verify_output "remove file" # a/aa/a21.txt
    verify_output "remove file" # a/aa/a22.txt
    # sometimes combined two notify into one
    verify_output "modify dir" # a/aa
    verify_output "modify dir" # a/aa
    verify_output "remove dir"  # a/aa
    verify_output "modify dir" # a
    verify_output "remove dir"  # a
}

test_case_insensitive()
{
    local file1="aBc"
    local file2="AbC"
    local dir1="def"
    local dir2="DEF"

    touch "$file1"
    echo "touch $file1 :"
    verify_output "add file" 

    # cannot create same name...
    #touch "$file2"
    #echo "touch $file2 :"
    #verify_output "add file" 

    # move self only notify twice:
    # no dest remove
    mv "$file1" "$file2"
    echo "move $file1 to $file2 :" 
    verify_output "modify file" 
    verify_output "remove file"
    verify_output "rename file" 
    #verify_output "add file"
    #verify_output "modify file"

    rm "$file2"
    echo "remove $file2 :"
    verify_output "remove file"

    mkdir "$dir1"
    echo "mkdir $dir1 :"
    verify_output "add dir"

    mv "$dir1" "$dir2"
    echo "move $dir1 to $dir2 :"
    verify_output "remove dir" 
    verify_output "rename dir" 

    rmdir "$dir2"
    echo "rmdir $dir2 :"
    verify_output "remove dir"
}
